@@ -23,6 +23,7 @@ import threading
 import re
 import mozrunner
 from mozrunner import global_settings
+from decorator import decorator
 
 from zc.testbrowser import real as browser
 from collective.anonymousbrowser import browser as abrowser
@@ -46,6 +47,20 @@ MOZ_USER_PROFILE = firefox_user_profile()
 if MOZ_USER_PROFILE:
     global_settings.MOZILLA_DEFAULT_PROFILE = MOZ_USER_PROFILE
 
+def exec_on_firefox_dir(*args):
+    def go_in_dir(f, *args, **kwargs):
+        ret, cwd = None, os.getcwd()
+        d = os.path.dirname(global_settings.firefoxBin)
+        os.chdir(d)
+        try:
+            ret = f(*args, **kwargs)
+            os.chdir(cwd)
+        except:
+            os.chdir(cwd)
+            raise
+        return ret
+    return decorator(go_in_dir)
+
 class FirefoxThread():
 
     def __init__(self, *args, **kwargs):
@@ -54,6 +69,7 @@ class FirefoxThread():
         self.FIREFOX = None
         self.FIREFOX_THREAD = None
 
+    @exec_on_firefox_dir
     def set_settings(self, *args, **kwargs):
         # setting no window bugs from ff
         if 'prefs' in kwargs:
@@ -62,11 +78,13 @@ class FirefoxThread():
         for setting in kwargs:
             setattr(global_settings, setting, kwargs[setting])
 
+    @exec_on_firefox_dir
     def get_instance(self, *args, **kwargs):
         if not self.FIREFOX:
             self.FIREFOX = mozrunner.get_moz_from_settings()
         return self.FIREFOX
 
+    @exec_on_firefox_dir()
     def start_ff(self, *args, **kwargs):
         if not self.FIREFOX:
             self.FIREFOX = self.get_instance(*args, **kwargs)
@@ -74,6 +92,7 @@ class FirefoxThread():
             time.sleep(3)
         return self.FIREFOX
 
+    @exec_on_firefox_dir
     def stop_ff(self, *args, **kwargs):
         if self.FIREFOX:
             self.FIREFOX.kill(signal.SIGKILL)
